# Generated by Django 5.0.3 on 2024-03-23 01:01

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("ptz_app", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="preset",
            name="thumbnail",
            field=models.BinaryField(),
        ),
    ]
