# Generated by Django 5.0.3 on 2024-03-26 17:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("ptz_app", "0002_alter_preset_thumbnail"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="camerasettings",
            name="aperture",
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="blue_gain",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="color_temperature",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="drc",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="gain",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="gamma",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="iris",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="noise2d",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="noise3d",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="red_gain",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="camerasettings",
            name="shutter",
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name="preset",
            name="focus",
            field=models.IntegerField(),
        ),
        migrations.AlterField(
            model_name="preset",
            name="pan",
            field=models.IntegerField(),
        ),
        migrations.AlterField(
            model_name="preset",
            name="tilt",
            field=models.IntegerField(),
        ),
        migrations.AlterField(
            model_name="preset",
            name="zoom",
            field=models.IntegerField(),
        ),
    ]
